#include "../args.h"
#include "../cmd.h"
#include "../config.h"
#include "../error.h"
#include "../log.h"
#include "../run.h"
#include "../url.h"
#include "../util.h"

#include <git2.h>
#include <git2/global.h>
#include <libintl.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>

#define _(x) gettext(x)

int pull_progress(const git_indexer_progress *stats, void *data) {
  bar(stats->indexed_objects, stats->total_objects);
  return 0;
}

bool cmd_pull(ctx_t *ctx, args_t *args) {
  char    *repo_url = NULL, *repo_root = NULL, *url = NULL;
  config_t repo_cfg = {.name = NULL};
  bool     ret      = false;

  for (size_t i = 0; i < args->count; i++) {
    if (NULL != args->list[i].name || eq(args->list[i].value, "pull"))
      continue;

    if (NULL == url) {
      url = args->list[i].value;
      continue;
    }

    error(_("Please specify only a single config name or a URL"));
    return false;
  }

  if (NULL == url) {
    error(_("Please specify a config name or a URL"));
    return false;
  }

  if (url_is_local(url)) {
    repo_root = url;
    goto COPY;
  }

  else if (url_is_name(url))
    repo_url = url_complete(url);

  else
    repo_url = url_ensure_protocol(url);

  if (NULL == repo_url) {
    switch (errno) {
    case UrlReqBadCode:
      error(_("Specified configuration is not found"));
      break;
    default:
      error(_("Failed to access the URL"));
      details(errch);
    }
    goto END;
  }

  rmrf(ctx->tmp_path);
  repo_root = ctx->tmp_path;

  git_libgit2_init();

  git_repository   *repo                      = NULL;
  git_clone_options opts                      = GIT_CLONE_OPTIONS_INIT;
  opts.fetch_opts.callbacks.transfer_progress = pull_progress;
  opts.fetch_opts.callbacks.payload           = NULL;

  info(_("Cloning %s"), repo_url);
  bar_init();

  if (git_clone(&repo, repo_url, repo_root, &opts) < 0) {
    const git_error *e = git_error_last();
    error(_("Failed to clone the %s:"), repo_url);
    details(e->message);
    bar_free();
    goto END;
  }

  bar_free();
  git_libgit2_shutdown();

COPY:
  if (chdir(repo_root) < 0) {
    error(_("Failed to chdir to %s"), repo_root);
    goto END;
  }

  if (!config_load(ctx, &repo_cfg, "mc.cfg")) {
    error(_("Failed to load the configuration file (mc.cfg):"));
    details(errch);
    goto END;
  }

  success(_("Loaded repository configuration"));
  config_print(&repo_cfg);

  if (!yesno(_("Do you want to continue?")))
    goto END;

  target_t *cur     = repo_cfg.t_first;
  int       counter = 0;

  info(_("Checking all the targets"));
  bar_init();

  while (NULL != cur && ++counter > 0) {
    if (!exists(cur->src)) {
      bar_free();
      error(_("Failed to access the source for the target \"%s\""), cur->name);
      goto END;
    }

    cur = cur->next;
    bar(counter, repo_cfg.t_len);
  }

  bar_free();
  success(_("All the target checks were successful"));

  cur = repo_cfg.t_first;
  info(_("Copying all the targets"));

  char **argv = NULL;
  int    argc = 0, indx = 0;

  while (cur && ++counter > 0) {
    target_print(cur);

    if (!yesno(_("Install the target?"))) {
      info(_("Skipping target"));
      goto NEXT;
    }

    if (!target_copy(cur, false)) {
      error(_("Failed to copy the target:"));
      details(errch);
      goto END;
    }

    argc += cur->require.s;
    if (0 == argc)
      goto NEXT;

    if (NULL == argv)
      argv = malloc(sizeof(char *) * argc);
    else
      argv = realloc(argv, sizeof(char *) * argc);

    for (int i = 0; i < cur->require.s; i++) {
      argv[indx] = cur->require.c[i];
      indx++;
    }

  NEXT:
    cur = cur->next;
  }

  if (NULL == argv) {
    ret = true;
    goto END;
  }

  argc++;
  argv       = realloc(argv, sizeof(char *) * argc);
  argv[indx] = NULL;
  ret        = true;

  info(_("Installing all the requirements"));
  if (!run_cmd_root("mp-install", argv)) {
    ret = false;
    error(_("Failed to run the mp-install command"));
    details(errch);
  }

  free(argv);

END:
  if (repo_cfg.name != NULL)
    config_free(&repo_cfg);
  free(repo_url);
  return ret;
}
